@@ -4,17 +4,14 @@ use warnings;
 use utf8;
 
 package Neo4j::Driver;
-# ABSTRACT: Perl implementation of the Neo4j Driver API
-$Neo4j::Driver::VERSION = '0.18';
+# ABSTRACT: Perl Neo4j driver for Bolt and HTTP
+$Neo4j::Driver::VERSION = '0.20';
 
 use Carp qw(croak);
 
 use URI 1.25;
-use Neo4j::Driver::Transport::HTTP;
 use Neo4j::Driver::Session;
 
-# The following packages are never used directly anywhere. We mention them
-# here so that a simple `use Neo4j::Driver;` will make them available.
 use Neo4j::Driver::Type::Node;
 use Neo4j::Driver::Type::Relationship;
 use Neo4j::Driver::Type::Path;
@@ -32,6 +29,7 @@ my %OPTIONS = (
 	ca_file => 'tls_ca',
 	cypher_filter => 'cypher_filter',
 	cypher_types => 'cypher_types',
+	net_module => 'net_module',
 	timeout => 'http_timeout',
 	tls => 'tls',
 	tls_ca => 'tls_ca',
@@ -60,11 +58,6 @@ sub new {
 		if (! $uri->scheme || $uri->scheme !~ m/^https?|bolt$/) {
 			croak sprintf "URI scheme '%s' unsupported; use 'http' or 'bolt'", $uri->scheme // "";
 		}
-		if ($uri->scheme eq 'bolt') {
-			eval { require Neo4j::Bolt; };
-			croak "URI scheme 'bolt' requires Neo4j::Bolt. Can't locate Neo4j/Bolt.pm in \@INC " .
-			      "(you may need to install the Neo4j::Bolt module) (\@INC contains: @INC)" if $@;
-		}
 		
 		$uri->host('localhost') unless $uri->host;
 		$uri->path('') if $uri->path_query eq '/';
@@ -121,23 +114,15 @@ sub session {
 	warnings::warnif deprecated => __PACKAGE__ . "->{die_on_error} is deprecated" unless $self->{die_on_error};
 	my %options = $self->_parse_options('session', ['database'], @options);
 	
-	my $transport;
-	if ($self->{uri}->scheme eq 'bolt') {
-		require Neo4j::Driver::Transport::Bolt;
-		$transport = Neo4j::Driver::Transport::Bolt->new($self);
-	}
-	else {
-		$transport = Neo4j::Driver::Transport::HTTP->new($self);
-		$transport->_connect($options{database});
-	}
 	$self->{session} = 1;
 	
-	return Neo4j::Driver::Session->new($transport);
+	my $session = Neo4j::Driver::Session->new($self);
+	return $session->_connect($options{database});
 }
 
 
 sub _parse_options {
-	my ($self, $context, $supported, @options) = @_;
+	my (undef, $context, $supported, @options) = @_;
 	
 	croak "Odd number of elements in $context options hash" if @options & 1;
 	my %options = @options;
@@ -157,6 +142,17 @@ sub close {
 }
 
 
+
+
+package # private
+        URI::bolt;
+
+use parent 'URI::_server';
+
+# The server methods need to be available for bolt: URI instances
+# even when the Neo4j-Bolt distribution is not installed.
+
+ 
 1;
 
 __END__
@@ -167,11 +163,11 @@ __END__
 
 =head1 NAME
 
-Neo4j::Driver - Perl implementation of the Neo4j Driver API
+Neo4j::Driver - Perl Neo4j driver for Bolt and HTTP
 
 =head1 VERSION
 
-version 0.18
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -309,6 +305,16 @@ These are subject to unannounced modification or removal in future
 versions. Expect your code to break if you depend upon these
 features.
 
+=head2 Bolt version 3 and 4
+
+There is now experimental support for Bolt protocol versions newer
+S<than 1> using L<Neo4j::Bolt> S<version 0.40> or newer. This allows
+connecting to S<Neo4j 4> using Bolt.
+
+This feature is currently undergoing testing. Until it becomes
+stable, please continue to use HTTP with S<Neo4j 4> in production
+environments.
+
 =head2 Parameter syntax conversion
 
  $driver->config(cypher_filter => 'params');
@@ -466,13 +472,21 @@ L<REST::Neo4p>
 
 =back
 
+=head1 ACKNOWLEDGEMENT
+
+Special thanks go to Mark A. Jensen (MAJENSEN). Without the
+inspiration of his L<REST::Neo4p>, this driver project I<probably>
+would never have been even gotten started. And without Mark's
+tremendous work on L<Neo4j::Bolt> and libneo4j-client, this
+driver I<certainly> would be a far cry from what it is today.
+
 =head1 AUTHOR
 
 Arne Johannessen <ajnn@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016-2020 by Arne Johannessen.
+This software is Copyright (c) 2016-2021 by Arne Johannessen.
 
 This is free software, licensed under:
 
