@@ -5,21 +5,27 @@ use utf8;
 
 package Neo4j::Driver::Transaction;
 # ABSTRACT: Logical container for an atomic unit of work
-$Neo4j::Driver::Transaction::VERSION = '0.18';
+$Neo4j::Driver::Transaction::VERSION = '0.20';
 
 use Carp qw(croak);
-our @CARP_NOT = qw(Neo4j::Driver::Session Neo4j::Driver);
+our @CARP_NOT = qw(
+	Neo4j::Driver::Session
+	Neo4j::Driver::Session::Bolt
+	Neo4j::Driver::Session::HTTP
+);
 use Scalar::Util qw(blessed);
 
-use Neo4j::Driver::StatementResult;
+use Neo4j::Driver::Result;
 
 
 sub new {
 	my ($class, $session) = @_;
 	
 	my $transaction = {
-		transport => $session->{transport},
-		open => 1,
+		cypher_filter => $session->{driver}->{cypher_filter},
+		net => $session->{net},
+		unused => 1,  # for HTTP only
+		closed => 0,
 		return_graph => 0,
 		return_stats => 1,
 	};
@@ -46,10 +52,10 @@ sub run {
 		@statements = ();
 	}
 	
-	my @results = $self->{transport}->run($self, @statements);
+	my @results = $self->{net}->_run($self, @statements);
 	
 	if (scalar @statements <= 1) {
-		my $result = $results[0] // Neo4j::Driver::StatementResult->new;
+		my $result = $results[0] // Neo4j::Driver::Result->new;
 		return wantarray ? $result->list : $result;
 	}
 	return wantarray ? @results : \@results;
@@ -75,25 +81,61 @@ sub _prepare {
 		$params = {@parameters};
 	}
 	
-	$self->{transport}->{return_graph} = $self->{return_graph};
-	$self->{transport}->{return_stats} = $self->{return_stats};
-	return $self->{transport}->prepare($self, $query, $params);
+	if ($self->{cypher_filter}) {
+		croak "Unimplemented cypher filter '$self->{cypher_filter}'" if $self->{cypher_filter} ne 'params';
+		if (defined $params) {
+			my @params_quoted = map {quotemeta} keys %$params;
+			my $params_re = join '|', @params_quoted, map {"`$_`"} @params_quoted;
+			$query =~ s/\{($params_re)}/\$$1/g;
+		}
+	}
+	
+	my $statement = [$query, $params // {}];
+	return $statement;
 }
 
 
-sub _explicit {
+
+
+package # private
+        Neo4j::Driver::Transaction::Bolt;
+use parent -norequire => 'Neo4j::Driver::Transaction';
+
+use Carp qw(croak);
+use Try::Tiny;
+
+
+sub _begin {
 	my ($self) = @_;
 	
-	$self->{transport}->begin($self);
+	croak 'Nested transactions unsupported in Bolt' if $self->{net}->{active_tx};
+	
+	$self->{bolt_txn} = $self->{net}->_new_tx;
+	$self->{net}->{active_tx} = 1;
+	$self->run('BEGIN') unless $self->{bolt_txn};
 	return $self;
 }
 
 
-sub _autocommit {
-	my ($self) = @_;
+sub _run_autocommit {
+	my ($self, $query, @parameters) = @_;
 	
-	$self->{transport}->autocommit($self);
-	return $self;
+	croak 'Nested transactions unsupported in Bolt' if $self->{net}->{active_tx};
+	
+	$self->{net}->{active_tx} = 1;  # run() requires an active tx
+	my $results;
+	try {
+		$results = $self->run($query, @parameters);
+	}
+	catch {
+		$self->{net}->{active_tx} = 0;
+		croak $_;
+	};
+	$self->{net}->{active_tx} = 0;
+	
+	return $results unless wantarray;
+	return $results->list if ref $query ne 'ARRAY';
+	return @$results;
 }
 
 
@@ -102,8 +144,14 @@ sub commit {
 	
 	croak 'Transaction already closed' unless $self->is_open;
 	
-	$self->{transport}->commit($self);
-	$self->{open} = 0;
+	if ($self->{bolt_txn}) {
+		$self->{bolt_txn}->commit;
+	}
+	else {
+		$self->run('COMMIT');
+	}
+	$self->{closed} = 1;
+	$self->{net}->{active_tx} = 0;
 }
 
 
@@ -112,15 +160,95 @@ sub rollback {
 	
 	croak 'Transaction already closed' unless $self->is_open;
 	
-	$self->{transport}->rollback($self);
-	$self->{open} = 0;
+	if ($self->{bolt_txn}) {
+		$self->{bolt_txn}->rollback;
+	}
+	else {
+		$self->run('ROLLBACK');
+	}
+	$self->{closed} = 1;
+	$self->{net}->{active_tx} = 0;
 }
 
 
 sub is_open {
 	my ($self) = @_;
 	
-	return $self->{open};
+	return 0 if $self->{closed};  # what is closed stays closed
+	return $self->{net}->{active_tx};
+}
+
+
+
+
+package # private
+        Neo4j::Driver::Transaction::HTTP;
+use parent -norequire => 'Neo4j::Driver::Transaction';
+
+use Carp qw(croak);
+
+# use 'rest' in place of broken 'meta', see neo4j #12306
+my $RESULT_DATA_CONTENTS = ['row', 'rest'];
+my $RESULT_DATA_CONTENTS_GRAPH = ['row', 'rest', 'graph'];
+
+
+sub _prepare {
+	my ($self, $query, @parameters) = @_;
+	
+	my $statement = $self->SUPER::_prepare($query, @parameters);
+	my ($cypher, $parameters) = @$statement;
+	
+	my $json = { statement => '' . $cypher };
+	$json->{resultDataContents} = $RESULT_DATA_CONTENTS;
+	$json->{resultDataContents} = $RESULT_DATA_CONTENTS_GRAPH if $self->{return_graph};
+	$json->{includeStats} = \1 if $self->{return_stats};
+	$json->{parameters} = $parameters if %$parameters;
+	
+	return $json;
+}
+
+
+sub _begin {
+	my ($self) = @_;
+	
+	# no-op for HTTP
+	return $self;
+}
+
+
+sub _run_autocommit {
+	my ($self, $query, @parameters) = @_;
+	
+	$self->{transaction_endpoint} = $self->{commit_endpoint};
+	$self->{transaction_endpoint} //= URI->new( $self->{net}->{endpoints}->{new_commit} )->path;
+	
+	return $self->run($query, @parameters);
+}
+
+
+sub commit {
+	my ($self) = @_;
+	
+	$self->_run_autocommit;
+}
+
+
+sub rollback {
+	my ($self) = @_;
+	
+	croak 'Transaction already closed' unless $self->is_open;
+	
+	$self->{net}->_request($self, 'DELETE') if $self->{transaction_endpoint};
+	$self->{closed} = 1;
+}
+
+
+sub is_open {
+	my ($self) = @_;
+	
+	return 0 if $self->{closed};
+	return 1 if $self->{unused};
+	return $self->{net}->_is_active_tx($self);
 }
 
 
@@ -138,7 +266,7 @@ Neo4j::Driver::Transaction - Logical container for an atomic unit of work
 
 =head1 VERSION
 
-version 0.18
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -171,7 +299,7 @@ Statements may be run lazily. Most of the time, you will not notice
 this, because the driver automatically waits for statements to
 complete at specific points to fulfill its contracts. If you require
 execution of a statement to have completed, you need to use the
-L<result|Neo4j::Driver::StatementResult>, for example by calling
+L<Result|Neo4j::Driver::Result>, for example by calling
 one of the methods C<fetch()>, C<list()> or C<summary()>.
 
 =head1 METHODS
@@ -191,12 +319,11 @@ After committing the transaction is closed and can no longer be used.
  $bool = $transaction->is_open;
 
 Report whether this transaction is still open, which means commit
-or rollback did not happen.
+or rollback has not happened and the transaction has not timed out.
 
-On HTTP connections, a transaction can timeout on the server due
-to inactivity. In this case, it may in fact be closed even though
-this method returns a truthy value. The Neo4j server default
-C<dbms.rest.transaction.idle_timeout> is 60 seconds.
+Bolt transactions by default have no timeout, while the default
+C<dbms.rest.transaction.idle_timeout> for HTTP transactions is
+S<60 seconds.>
 
 =head2 rollback
 
@@ -211,7 +338,7 @@ used.
 
  $result = $transaction->run($query, %params);
 
-Run a statement and return the L<StatementResult|Neo4j::Driver::StatementResult>.
+Run a statement and return the L<Result|Neo4j::Driver::Result>.
 This method takes an optional set of parameters that will be injected
 into the Cypher statement by Neo4j. Using parameters is highly
 encouraged: It helps avoid dangerous Cypher injection attacks and
@@ -351,9 +478,9 @@ The ability to disable the statistics may be removed in future.
    ...
  }
 
-The Neo4j HTTP API supports a "graph" results data format. This driver
-exposes this feature to the client and will continue to do so, but
-the interface is not yet finalised.
+The Neo4j HTTP JSON API supports a "graph" results data format.
+This driver exposes this feature to the client and will continue
+to do so, but the interface is not yet finalised.
 
 =head1 SEE ALSO
 
@@ -361,7 +488,7 @@ the interface is not yet finalised.
 
 =item * L<Neo4j::Driver>
 
-=item * L<Neo4j::Driver::B<StatementResult>>
+=item * L<Neo4j::Driver::B<Result>>
 
 =item * Equivalent documentation for the official Neo4j drivers:
 L<Transaction (Java)|https://neo4j.com/docs/api/java-driver/current/index.html?org/neo4j/driver/Transaction.html>,
@@ -379,7 +506,7 @@ Arne Johannessen <ajnn@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016-2020 by Arne Johannessen.
+This software is Copyright (c) 2016-2021 by Arne Johannessen.
 
 This is free software, licensed under:
 
